@@ -118,13 +118,20 @@ void main(int argc, char *argv[], char *environ[])
  struct passwd *j;
  char **env;
 
- /* correct SHELL environment variable and learn who we are MM */ 
- for (env = environ; *env; env++)
-    if (!strncmp (*env, "SHELL=", 6)) {
- 	if ((ptr = strstr(*env, "/lshells")) != NULL)
-           memmove (ptr, ptr + 8, strlen(*env) - 8);
-        shell = *env + 6;
-    }
+  /* correct SHELL environment variable and learn who we are MM */ 
+
+  /* (HS) Behaviour like the `old' lshell, but w/o destroying the
+   * environment.  (The `old' lshell destroyed the TERM environment,
+   * under some cirumstances.  */
+  ptr = getenv("SHELL");
+  if (ptr) {
+	shell = strdup(ptr);
+	if (ptr = strstr(shell, "/lshells"))  {
+		i = strlen("/lshells");
+		memmove(ptr, ptr + i, 1 + strlen(shell) - i - (ptr - shell));
+	}
+	setenv("SHELL", shell, 1);
+  } else shell = NULL;
     
  if (argv[0][0]=='-')
  { /* We are a login shell */
